package sun.nio.fs;

import java.nio.file.AtomicMoveNotSupportedException;
import java.util.concurrent.ExecutionException;
import java.io.IOException;
import java.nio.file.DirectoryNotEmptyException;
import java.nio.file.LinkPermission;
import java.nio.file.FileAlreadyExistsException;
import com.sun.nio.file.ExtendedCopyOption;
import java.nio.file.LinkOption;
import java.nio.file.StandardCopyOption;
import java.nio.file.CopyOption;

class WindowsFileCopy
{
    private WindowsFileCopy() {
    }
    
    static void copy(final WindowsPath windowsPath, final WindowsPath windowsPath2, CopyOption... array) throws IOException {
        boolean n = false;
        boolean n2 = false;
        boolean n3 = true;
        boolean n4 = false;
        for (CopyOption copyOption : array) {
            if (copyOption == StandardCopyOption.REPLACE_EXISTING) {
                n = true;
            }
            else if (copyOption == LinkOption.NOFOLLOW_LINKS) {
                n3 = false;
            }
            else if (copyOption == StandardCopyOption.COPY_ATTRIBUTES) {
                n2 = true;
            }
            else if (copyOption == ExtendedCopyOption.INTERRUPTIBLE) {
                n4 = true;
            }
            else {
                if (copyOption == null) {
                    throw new NullPointerException();
                }
                throw new UnsupportedOperationException("Unsupported copy option");
            }
        }
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            windowsPath.checkRead();
            windowsPath2.checkWrite();
        }
        WindowsFileAttributes attributes = null;
        WindowsFileAttributes attributes2 = null;
        long openForReadAttributeAccess = 0L;
        try {
            openForReadAttributeAccess = windowsPath.openForReadAttributeAccess((boolean)n3);
        }
        catch (WindowsException ex) {
            ex.rethrowAsIOException(windowsPath);
        }
        try {
            try {
                attributes = WindowsFileAttributes.readAttributes(openForReadAttributeAccess);
            }
            catch (WindowsException ex2) {
                ex2.rethrowAsIOException(windowsPath);
            }
            try {
                long openForReadAttributeAccess2 = windowsPath2.openForReadAttributeAccess(false);
                try {
                    attributes2 = WindowsFileAttributes.readAttributes(openForReadAttributeAccess2);
                    if (WindowsFileAttributes.isSameFile(attributes, attributes2)) {
                        WindowsNativeDispatcher.CloseHandle(openForReadAttributeAccess2);
                        return;
                    }
                    if (n == 0) {
                        throw new FileAlreadyExistsException(windowsPath2.getPathForExceptionMessage());
                    }
                }
                finally {
                    WindowsNativeDispatcher.CloseHandle(openForReadAttributeAccess2);
                }
            }
            catch (WindowsException ex3) {}
        }
        finally {
            WindowsNativeDispatcher.CloseHandle(openForReadAttributeAccess);
        }
        if (securityManager != null && attributes.isSymbolicLink()) {
            securityManager.checkPermission(new LinkPermission("symbolic"));
        }
        final String win32Path = asWin32Path(windowsPath);
        final String win32Path2 = asWin32Path(windowsPath2);
        if (attributes2 != null) {
            try {
                if (attributes2.isDirectory() || attributes2.isDirectoryLink()) {
                    WindowsNativeDispatcher.RemoveDirectory(win32Path2);
                }
                else {
                    WindowsNativeDispatcher.DeleteFile(win32Path2);
                }
            }
            catch (WindowsException ex4) {
                if (attributes2.isDirectory() && (ex4.lastError() == 145 || ex4.lastError() == 183)) {
                    throw new DirectoryNotEmptyException(windowsPath2.getPathForExceptionMessage());
                }
                ex4.rethrowAsIOException(windowsPath2);
            }
        }
        if (!attributes.isDirectory() && !attributes.isDirectoryLink()) {
            final int n5 = (windowsPath.getFileSystem().supportsLinks() && n3 == 0) ? 2048 : 0;
            if (!(n4 == 0)) {
                WindowsFileCopy.1 o = new Cancellable() {
                    public int cancelValue() {
                        return 1;
                    }
                    
                    public void implRun() throws IOException {
                        try {
                            WindowsNativeDispatcher.CopyFileEx(win32Path, win32Path2, n5, this.addressToPollForCancel());
                        }
                        catch (WindowsException ex) {
                            ex.rethrowAsIOException(windowsPath, windowsPath2);
                        }
                    }
                };
                try {
                    Cancellable.runInterruptibly(o);
                }
                catch (ExecutionException ex10) {
                    Throwable cause = ex10.getCause();
                    if ((cause instanceof IOException)) {
                        throw (IOException)cause;
                    }
                    throw new IOException(cause);
                }
            }
            else {
                try {
                    WindowsNativeDispatcher.CopyFileEx(win32Path, win32Path2, n5, 0L);
                }
                catch (WindowsException ex9) {
                    ex9.rethrowAsIOException(windowsPath, windowsPath2);
                }
            }
            if (!(n2 == 0)) {
                try {
                    copySecurityAttributes(windowsPath, windowsPath2, (boolean)n3);
                }
                catch (IOException ex11) {}
            }
            return;
        }
        else {
            try {
                if (attributes.isDirectory()) {
                    WindowsNativeDispatcher.CreateDirectory(win32Path2, 0L);
                }
                else {
                    WindowsNativeDispatcher.CreateSymbolicLink(win32Path2, WindowsPath.addPrefixIfNeeded(WindowsLinkSupport.readLink(windowsPath)), 1);
                }
            }
            catch (WindowsException ex5) {
                ex5.rethrowAsIOException(windowsPath2);
            }
            if (!(n2 == 0)) {
                WindowsFileAttributeViews.Dos dosView = WindowsFileAttributeViews.createDosView(windowsPath2, false);
                try {
                    dosView.setAttributes(attributes);
                }
                catch (IOException ex6) {
                    if (attributes.isDirectory()) {
                        try {
                            WindowsNativeDispatcher.RemoveDirectory(win32Path2);
                        }
                        catch (WindowsException ex7) {}
                    }
                }
                try {
                    copySecurityAttributes(windowsPath, windowsPath2, (boolean)n3);
                }
                catch (IOException ex8) {}
            }
            return;
        }
    }
    
    static void move(WindowsPath windowsPath, WindowsPath windowsPath2, CopyOption... array) throws IOException {
        boolean n = false;
        boolean n2 = false;
        for (CopyOption copyOption : array) {
            if (copyOption == StandardCopyOption.ATOMIC_MOVE) {
                n = true;
            }
            else if (copyOption == StandardCopyOption.REPLACE_EXISTING) {
                n2 = true;
            }
            else if (copyOption != LinkOption.NOFOLLOW_LINKS) {
                if (copyOption == null) {
                    throw new NullPointerException();
                }
                throw new UnsupportedOperationException("Unsupported copy option");
            }
        }
        if (System.getSecurityManager() != null) {
            windowsPath.checkWrite();
            windowsPath2.checkWrite();
        }
        String win32Path = asWin32Path(windowsPath);
        String win32Path2 = asWin32Path(windowsPath2);
        if (!(n == 0)) {
            try {
                WindowsNativeDispatcher.MoveFileEx(win32Path, win32Path2, 1);
            }
            catch (WindowsException ex14) {
                if (ex14.lastError() == 17) {
                    throw new AtomicMoveNotSupportedException(windowsPath.getPathForExceptionMessage(), windowsPath2.getPathForExceptionMessage(), ex14.errorString());
                }
                ex14.rethrowAsIOException(windowsPath, windowsPath2);
            }
            return;
        }
        WindowsFileAttributes attributes = null;
        WindowsFileAttributes attributes2 = null;
        long openForReadAttributeAccess = 0L;
        try {
            openForReadAttributeAccess = windowsPath.openForReadAttributeAccess(false);
        }
        catch (WindowsException ex) {
            ex.rethrowAsIOException(windowsPath);
        }
        try {
            try {
                attributes = WindowsFileAttributes.readAttributes(openForReadAttributeAccess);
            }
            catch (WindowsException ex2) {
                ex2.rethrowAsIOException(windowsPath);
            }
            try {
                long openForReadAttributeAccess2 = windowsPath2.openForReadAttributeAccess(false);
                try {
                    attributes2 = WindowsFileAttributes.readAttributes(openForReadAttributeAccess2);
                    if (WindowsFileAttributes.isSameFile(attributes, attributes2)) {
                        WindowsNativeDispatcher.CloseHandle(openForReadAttributeAccess2);
                        return;
                    }
                    if (n2 == 0) {
                        throw new FileAlreadyExistsException(windowsPath2.getPathForExceptionMessage());
                    }
                }
                finally {
                    WindowsNativeDispatcher.CloseHandle(openForReadAttributeAccess2);
                }
            }
            catch (WindowsException ex3) {}
        }
        finally {
            WindowsNativeDispatcher.CloseHandle(openForReadAttributeAccess);
        }
        if (attributes2 != null) {
            try {
                if (attributes2.isDirectory() || attributes2.isDirectoryLink()) {
                    WindowsNativeDispatcher.RemoveDirectory(win32Path2);
                }
                else {
                    WindowsNativeDispatcher.DeleteFile(win32Path2);
                }
            }
            catch (WindowsException ex4) {
                if (attributes2.isDirectory() && (ex4.lastError() == 145 || ex4.lastError() == 183)) {
                    throw new DirectoryNotEmptyException(windowsPath2.getPathForExceptionMessage());
                }
                ex4.rethrowAsIOException(windowsPath2);
            }
        }
        try {
            WindowsNativeDispatcher.MoveFileEx(win32Path, win32Path2, 0);
            return;
        }
        catch (WindowsException ex5) {
            if (ex5.lastError() != 17) {
                ex5.rethrowAsIOException(windowsPath, windowsPath2);
            }
            if (!attributes.isDirectory() && !attributes.isDirectoryLink()) {
                try {
                    WindowsNativeDispatcher.MoveFileEx(win32Path, win32Path2, 2);
                }
                catch (WindowsException ex12) {
                    ex12.rethrowAsIOException(windowsPath, windowsPath2);
                }
                try {
                    copySecurityAttributes(windowsPath, windowsPath2, false);
                }
                catch (IOException ex13) {}
                return;
            }
            assert !(!attributes.isDirectoryLink());
            try {
                if (attributes.isDirectory()) {
                    WindowsNativeDispatcher.CreateDirectory(win32Path2, 0L);
                }
                else {
                    WindowsNativeDispatcher.CreateSymbolicLink(win32Path2, WindowsPath.addPrefixIfNeeded(WindowsLinkSupport.readLink(windowsPath)), 1);
                }
            }
            catch (WindowsException ex6) {
                ex6.rethrowAsIOException(windowsPath2);
            }
            WindowsFileAttributeViews.Dos dosView = WindowsFileAttributeViews.createDosView(windowsPath2, false);
            try {
                dosView.setAttributes(attributes);
            }
            catch (IOException ex7) {
                try {
                    WindowsNativeDispatcher.RemoveDirectory(win32Path2);
                }
                catch (WindowsException ex8) {}
                throw ex7;
            }
            try {
                copySecurityAttributes(windowsPath, windowsPath2, false);
            }
            catch (IOException ex9) {}
            try {
                WindowsNativeDispatcher.RemoveDirectory(win32Path);
            }
            catch (WindowsException ex10) {
                try {
                    WindowsNativeDispatcher.RemoveDirectory(win32Path2);
                }
                catch (WindowsException ex11) {}
                if (ex10.lastError() == 145 || ex10.lastError() == 183) {
                    throw new DirectoryNotEmptyException(windowsPath2.getPathForExceptionMessage());
                }
                ex10.rethrowAsIOException(windowsPath);
            }
            return;
        }
    }
    
    private static String asWin32Path(WindowsPath windowsPath) throws IOException {
        try {
            return windowsPath.getPathForWin32Calls();
        }
        catch (WindowsException ex) {
            ex.rethrowAsIOException(windowsPath);
            return null;
        }
    }
    
    private static void copySecurityAttributes(WindowsPath windowsPath, WindowsPath windowsPath2, boolean b) throws IOException {
        String finalPath = WindowsLinkSupport.getFinalPath(windowsPath, b);
        WindowsSecurity.Privilege enablePrivilege = WindowsSecurity.enablePrivilege("SeRestorePrivilege");
        try {
            int n = 7;
            NativeBuffer fileSecurity = WindowsAclFileAttributeView.getFileSecurity(finalPath, n);
            try {
                try {
                    WindowsNativeDispatcher.SetFileSecurity(windowsPath2.getPathForWin32Calls(), n, fileSecurity.address());
                }
                catch (WindowsException ex) {
                    ex.rethrowAsIOException(windowsPath2);
                }
            }
            finally {
                fileSecurity.release();
            }
        }
        finally {
            enablePrivilege.drop();
        }
    }
}
